// LuaLanguageInfo.h : Declaration of the CLuaLanguageInfo

#pragma once
#include "resource.h"       // main symbols

#include "LuaLite.h"
#include <vector>

// The language info service ID: {04DA40AA-AE4B-49B4-8033-CF3D0BCEE2EC}
extern GUID const SID_LuaLanguageInfo;

// CLuaLanguageInfo

class ATL_NO_VTABLE CLuaLanguageInfo : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CLuaLanguageInfo>, 
	public IVsLanguageInfo, 
	public IVsProvideColorableItems
{
public:
	CLuaLanguageInfo()
	{
	}

BEGIN_COM_MAP( CLuaLanguageInfo )
	COM_INTERFACE_ENTRY( IVsLanguageInfo )
	COM_INTERFACE_ENTRY( IVsProvideColorableItems )
END_COM_MAP()

	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return CreateColourables();
	}
	
	void FinalRelease() 
	{
	}

public:
	// IVsLanguageInfo
	STDMETHODIMP GetLanguageName( BSTR *name );
	STDMETHODIMP GetFileExtensions( BSTR *extensions );
	STDMETHODIMP GetColorizer( IVsTextLines *buffer, IVsColorizer **colouriser );
	STDMETHODIMP GetCodeWindowManager( IVsCodeWindow *window, IVsCodeWindowManager **manager );

	// IVsProvideColorableItems
	STDMETHODIMP GetItemCount( int* count );
	STDMETHODIMP GetColorableItem( int index, IVsColorableItem** item );

private:
	HRESULT CreateColourables();

	std::vector< CAdapt< CComPtr< IVsColorableItem > > > m_colourables;
};

// CLuaColourableItem

class ATL_NO_VTABLE CLuaColourableItem : 
	public CComObjectRootEx<CComSingleThreadModel>, 
	public CComCoClass<CLuaColourableItem>, 
	public IVsColorableItem
{
public:
	CLuaColourableItem() 
	  : m_name( L"Unknown" ), 
		m_foreground( CI_SYSPLAINTEXT_FG ),
		m_background( CI_SYSPLAINTEXT_BK ), 
		m_fontFlags( FF_DEFAULT )
	{
	}

BEGIN_COM_MAP( CLuaColourableItem )
	COM_INTERFACE_ENTRY( IVsColorableItem )
END_COM_MAP()

	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}
	
	void FinalRelease() 
	{
	}

public:
	void Create( COLORINDEX foreground, COLORINDEX background, DWORD fontFlags, std::wstring const& name );

	// IVsColorableItem
	STDMETHODIMP GetDefaultColors( COLORINDEX* foreground, COLORINDEX* background );
	STDMETHODIMP GetDefaultFontFlags( DWORD* fontFlags );
	STDMETHODIMP GetDisplayName( BSTR *name );

private:
	std::wstring m_name;
	COLORINDEX m_foreground, m_background;
	DWORD m_fontFlags;
};
