// LuaLitePackage.cpp : Implementation of CLuaLitePackage

#include "stdafx.h"
#include "LuaLitePackage.h"
#include "LuaLanguageInfo.h"
#include <proffserv.h>


// CLuaLitePackage

STDMETHODIMP CLuaLitePackage::SetSite( IServiceProvider *serviceProvider )
{
	// check for null arguments
	if( !serviceProvider )
		return E_INVALIDARG;

	// get a proffer service
	CComPtr<IProfferService> profferService;
    HRESULT hr = serviceProvider->QueryService( 
		SID_SProfferService, IID_IProfferService, reinterpret_cast<void**>( &profferService ) 
	);
	if( FAILED( hr ) )
		return hr;

	// proffer our stuff
	DWORD cookie;
	hr = profferService->ProfferService( SID_LuaLanguageInfo, this, &cookie );
	if( FAILED( hr ) )
		return hr;

	// done ok
	return S_OK;
}

STDMETHODIMP CLuaLitePackage::QueryClose( BOOL *canClose )
{
	// check for null arguments
	if( !canClose )
		return E_INVALIDARG;

	// we always can
	*canClose = TRUE;

	// done ok
	return S_OK;
}

STDMETHODIMP CLuaLitePackage::Close()
{
	// done ok
	return S_OK;
}

STDMETHODIMP CLuaLitePackage::GetAutomationObject( LPCOLESTR /*propName*/, IDispatch** /*dispatch*/ )
{
	return E_NOTIMPL;
}

STDMETHODIMP CLuaLitePackage::CreateTool( REFGUID /*persistenceSlot*/ )
{
	return E_NOTIMPL;
}

STDMETHODIMP CLuaLitePackage::ResetDefaults( PKGRESETFLAGS /*flags*/ )
{
	return E_NOTIMPL;
}

STDMETHODIMP CLuaLitePackage::GetPropertyPage( REFGUID /*pageID*/, VSPROPSHEETPAGE* /*page*/ )
{
	return E_NOTIMPL;
}

STDMETHODIMP CLuaLitePackage::QueryService( REFGUID serviceID,  REFIID riid,  void **object )
{
	// check for null arguments
	if( !object )
		return E_INVALIDARG;

	// check for our service
	if( IsEqualGUID( serviceID, SID_LuaLanguageInfo ) )
	{
		// create a language service
		CComPtr<IUnknown> service;
		HRESULT hr = CLuaLanguageInfo::CreateInstance( &service );
		if( FAILED( hr ) )
			return E_UNEXPECTED;

		// store it
		hr = service->QueryInterface( riid, object );
		if( FAILED( hr ) )
			return E_NOINTERFACE;

		// done
		return S_OK;
	}
	else
	{
		// no service
		return E_NOINTERFACE;
	}
}

STDMETHODIMP CLuaLitePackage::get_IdBmpSplash( UINT* bmpID )
{
	if( !bmpID )
		return E_POINTER;

	*bmpID = IDB_SPLASH;
	return S_OK;
}

STDMETHODIMP CLuaLitePackage::get_OfficialName( BSTR* officialName )
{
	if( !officialName )
		return E_POINTER;

	*officialName = SysAllocString( L"Lua 5.0 Syntax Highlighting" );
	return S_OK;
}

STDMETHODIMP CLuaLitePackage::get_ProductID( BSTR* productID )
{
	if( !productID )
		return E_POINTER;

	*productID = SysAllocString( L"lualite-1.0.2" );
	return S_OK;
}

STDMETHODIMP CLuaLitePackage::get_ProductDetails( BSTR* productDetails )
{
	if( !productDetails )
		return E_POINTER;

	*productDetails = SysAllocString( 
		L"To get the latest version of lualite please visit the lualite webpage at http://www.sjbrown.co.uk/lualite.html. "
		L"Bug reports or suggestions are welcome at si@sjbrown.co.uk."
	);
	return S_OK;
}

STDMETHODIMP CLuaLitePackage::get_IdIcoLogoForAboutbox( UINT* bmpID )
{
	if( !bmpID )
		return E_POINTER;

	*bmpID = IDI_LOGO;
	return S_OK;
}
