#pragma once

#include <cwchar>
#include <boost/scoped_array.hpp>
#include <string>
#include <set>

enum LuaScannerType
{
	LUA_TEXT = 1, 
	LUA_IDENTIFIER, 
	LUA_COMMENT, 
	LUA_KEYWORD, 
	LUA_NUMBER, 
	LUA_STRING, 
	LUA_OPERATOR, 
};

class LuaScanner
{
public:
	LuaScanner();

	int ScanBuffer( wchar_t const* buffer, int length, int state );
	int const* GetTypes() const { return m_types.get(); }

private:
	void Scan();

	void ScanText();
	void ScanCommentBlock();
	void ScanDoubleQuoteString();
	void ScanSingleQuoteString();
	void ScanBracketString();

	void ExtractIdentifier();
	void ExtractComment();
	void ExtractNumber();
	void ExtractText();

	bool IsValid() const;
	bool IsLetter() const;				//!< Matches regex [a-zA-Z_]
	bool IsDigit() const;				//!< Matches regex [0-9]
	bool IsSignedDigit() const;			//!< Matches regex \-[0-9]
	bool IsCommentBlock() const;		//!< Matches --[[
	bool IsDoubleBracketStart() const;	//!< Matches [[
	bool IsDoubleBracketEnd() const;	//!< Matches ]]
	bool IsComment() const;				//!< Matches --
	bool IsSingleQuote() const;			//!< Matches '
	bool IsDoubleQuote() const;			//!< Matches "
	bool IsEscapedSingleQuote() const;	//!< Matches \'
	bool IsEscapedDoubleQuote() const;	//!< Matches \"
	bool IsEscapedEscape() const;		//!< Matches \\ 

	bool Matches( wchar_t const* str ) const;

	void Advance( int type, int count = 1 );

	enum STATE_TYPE
	{
		STATE_DEFAULT, 
		STATE_COMMENT_BLOCK, 
		STATE_STRING_SQ, 
		STATE_STRING_DQ, 
		STATE_STRING_BK
	};

	std::set<std::wstring> m_keywords;

	std::set<std::wstring> m_operators;
	int m_maxOperatorLength;

	wchar_t const* m_buffer;
	int m_length;
	
	STATE_TYPE m_state;
	int m_level;
	int m_pos;

	boost::scoped_array<int> m_types;
	int m_typesLength;
};
