// LuaWindowManager.cpp : Implementation of CLuaWindowManager

#include "stdafx.h"
#include "LuaWindowManager.h"
#include "LuaFilter.h"

// CLuaWindowManager

STDMETHODIMP CLuaWindowManager::AddAdornments()
{
	// set up the primary view
    CComPtr<IVsTextView> view;
    HRESULT hr = m_window->GetPrimaryView( &view );
	if( SUCCEEDED( hr ) )
        OnNewView( view );

	// set up the secondary view
	view.Release();
    hr = m_window->GetSecondaryView( &view );
	if( SUCCEEDED( hr ) )
        OnNewView( view );

	// done
	return S_OK;
}

STDMETHODIMP CLuaWindowManager::RemoveAdornments()
{
	// release the targets
	while( !m_targets.empty() )
	{
		// get the last one
		IOleCommandTarget* target = m_targets.back().m_T;
		CLuaFilter* parent = static_cast<CLuaFilter*>( target );

		// detach it
		parent->Detach();

		// pop it
		m_targets.pop_back();
	}

	// release the window
	m_window.Release();

	// done
	return S_OK;
}

STDMETHODIMP CLuaWindowManager::OnNewView( IVsTextView* view )
{
	// create a command filter for this view
	CComPtr<IOleCommandTarget> filter;
	HRESULT hr = CLuaFilter::CreateInstance( &filter );
	if( FAILED( hr ) )
		return hr;

	// attach it
	CLuaFilter* parent = static_cast<CLuaFilter*>( filter.p );
	hr = parent->Attach( view );
	if( FAILED( hr ) )
		return hr;

	// add to our list
	m_targets.push_back( filter );

	// done
	return S_OK;
}
